import json

from stumpf.tama import ChibiTama, listen


task_query = """
SELECT answer FROM assignment
    WHERE task_id = '{task_id}'
"""

# at least 3 / 5 or (2 / 3) agreement required
THRESHOLD = 0.66

# aggreement can only be computed if more than this many assignments are complete
MINIMUM_ASSIGNMENTS = 2

ARITY_APP_ID = '2d755948-af16-40bf-846e-0ce885a43b53'


def adjudicate_task(engine, task_id):
    # initialize activity detection count object
    #   activity name ->
    #      [ <# of 0 detections>, <# of 1 detections>, <# of 2+ detections> ]
    activities = {}
    for row in engine.execute(task_query.format(task_id=task_id)):
        if row[0] is None:
            continue
        for activity, value in row[0].items():
            if activity not in activities:
                activities[activity] = [0, 0, 0]
            if value is not None:
                activities[activity][value] += 1

    # adjudicate answers (contains only activities with agreement)
    #   activity -> 0, 1, or 2
    adjudicated = {}
    for activity, counts in activities.items():
        total = sum(counts)
        if not total or len(counts) < MINIMUM_ASSIGNMENTS:
            continue
        for value, count in enumerate(counts):
            if count / total >= THRESHOLD:
                adjudicated[activity] = value
                break

    # post answer back to the task
    engine.execute(f"UPDATE task SET answer = '{json.dumps(adjudicated)}' WHERE id = '{task_id}'")
    return adjudicated


class ArityChibiTama(ChibiTama):
    @listen('task:UPDATE')
    def adjudicate_arity_task(self, channel, new=None, old=None, **kwargs):
        if new['annotation_app_id'] != ARITY_APP_ID:
            self.log.debug('Skipping processing of non-arity app')
            return

        if old['status'] != 'Reviewable' and new['status'] == 'Reviewable':
            self.log.info(f'Adjudicating task {new["id"]}')
            answer = adjudicate_task(self.conn, new['id'])
            self.log.debug(f'Task {new["id"]} -> {json.dumps(answer)}')
        else:
            self.log.debug(f'Skipping processing of task in status {new["status"]}')


# For manual execution:
def adjudicate_all_tasks(engine):
    tasks = engine.execute(f"SELECT id FROM task WHERE annotation_app_id = '{ARITY_APP_ID}'")
    count = 0
    for task in tasks:
        count += 1
        adjudicate_task(engine, task['id'])
    return count


if __name__ == '__main__':
    import os
    from sqlalchemy import create_engine

    uri = os.environ['STUMPF_POSTGRES_URI']
    engine = create_engine(uri)
    count = adjudicate_all_tasks(engine)
    print(f'Processed {count} tasks')
